% =========================================================================
% ICA4SPM toolbox
%
% - IMM, Technical University of Denmark
%
% - version 1.0 
% T. Bjerre, J. Henriksen†, C.H. Nielsen, P.M. Rasmussen
%                    L.K. Hansen, K.H. Madsen
%
% 
%
% Bibtex reference:
%
%  @inproceedings{biosignals2009,
%    author = {Bjerre, T. and Henriksen, J. and Rasmussen, P.M. and Nielsen, C.H. and Hansen, L.K. and Madsen, K.H },
%    title = {Unified ICA-SPM analysis of f{MRI} experiments - Implementation of an {ICA} graphical user interface for the {SPM} pipeline},
%    booktitle = {Proceedings of BIOSTEC - BIOSIGNALS 2009 Conference},
%    year = {2009},}
%
%
% Change log:
% 11 apr 2010 - PMR: User must now supply a mask specifying which voxels to include in the analysis. Minor changes.
% 4 may 2011 - PMR: Minor revisions.
% 
% =========================================================================
close all, clear all, clc
warning off
global h_main ica st

% Setup ica structure as main variable:
ica = struct('TR',[],'S',[],'Structural',[]);

% Design main window:
h_main.gui = figure('units','normalized',...
    'Position',[0.01 0.05 .23 .9],...
    'menubar','none',...
    'name', 'ica4spm');

h_main.settings.panel = uipanel('units', 'normalized',...
    'Position', [.05 .5 .9 .48],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Settings');

h_main.settings.load = uicontrol('style', 'pushbutton',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.1 .88 .8 .1],...
    'string', 'Select data',...
    'CallBack',['ica.FileNames = spm_select(Inf,''image'');'...
    'ica.TR = inputdlg(''Set Repetition Time [s]:'');']);

h_main.settings.pca.check = uicontrol('style', 'checkbox',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.1 .75 .8 .1],...
    'string', '   Perform PCA',...
    'BackgroundColor',[.8 .8 .8],...
    'callback','icaonoff',...
    'tooltipstring','Check to perform Principal Component Analysis');
h_main.settings.pca.mask.text = uicontrol('style', 'text',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.15 .7 .30 .05],...
    'string', 'Mask',...
    'HorizontalAlignment','left',...
    'Enable','off',...
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Choose mask');
h_main.settings.pca.mask.methods = strvcat('User specified');
h_main.settings.pca.mask.method = uicontrol('style', 'popupmenu',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.4 .71 .5 .05],...
    'string', h_main.settings.pca.mask.methods,...
    'Enable','off',...
    'Callback','tauonoff',...
    'tooltipstring','Choose mask');

h_main.settings.ica.check = uicontrol('style', 'checkbox',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.1 .5 .8 .1],...
    'string', '   Perform ICA',...
    'BackgroundColor',[.8 .8 .8],...
    'CallBack','icaonoff',...
    'tooltipstring','Check to perform Independent Component Analysis');
h_main.settings.ica.algorithm.text = uicontrol('style', 'text',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.15 .45 .30 .05],...
    'string', 'Algorithm',...
    'Enable','off',...
    'HorizontalAlignment','left',...    
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Choose algorithm for ICA');
h_main.settings.ica.algorithm.methods = strvcat('Molgedey-Schuster',...
    'Maximum Likelihood','FastICA','Jade');
h_main.settings.ica.algorithm.method = uicontrol('style', 'popupmenu',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.4 .46 .5 .05],...
    'string', h_main.settings.ica.algorithm.methods,...
    'Enable','off',...
    'Callback','tauonoff',...
    'tooltipstring','Choose algorithm for ICA');
h_main.settings.ica.tau.check = uicontrol('style', 'checkbox',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.25 .35 .5 .05],...
    'string', 'Specify tau manually',...
    'BackgroundColor',[.8 .8 .8],...
    'Enable','off',...
    'Callback','tauonoff',...
    'tooltipstring','Check to manually select a value of \tau');
h_main.settings.ica.tau.set = uicontrol('style', 'edit',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.75 .35 .15 .05],...
    'string', '',...
    'Enable','off',...
    'tooltipstring','Value for tau',...
    'enable','off');
h_main.settings.ica.types = strvcat('Temporal','Spatial');
h_main.settings.ica.type.text = uicontrol('style', 'text',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.15 .25 .30 .05],...
    'string', 'Type',...
    'Enable','off',...
    'HorizontalAlignment','left',...
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Choose type of ICA analysis');
h_main.settings.ica.type.method = uicontrol('style', 'popupmenu',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.4 .26 .5 .05],...
    'Enable','off',...
    'string', h_main.settings.ica.types,...
    'tooltipstring','Choose type of ICA analysis');
h_main.settings.ica.bic.check = uicontrol('style', 'checkbox',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.15 .15 .6 .05],...
    'string', 'Perform BIC model selection',...
    'Enable','off',...
    'Value',0,...
    'CallBack','icaonoff',...
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Perform BIC model selection');

h_main.settings.ica.ICs.text = uicontrol('style', 'text',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [0.15 .05 .54 .05],...
    'string', 'Specify number of ICs',...
    'Enable','off',...
    'HorizontalAlignment','left',...    
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Specify number of independent components');
h_main.settings.ica.ICs.set = uicontrol('style', 'edit',...
    'parent', h_main.settings.panel,...
    'units', 'normalized',...
    'position', [.7 .05 .2 .05],...
    'string', '20',...
    'Enable','off',...
    'CallBack','ica.ICs = str2double(get(h_main.settings.ica.ICs.set,''String''));',...
    'tooltipstring','Specify number of independent components');
ica.ICs = str2double(get(h_main.settings.ica.ICs.set,'String'));

st.fig = [];
h_main.run = uicontrol('style', 'pushbutton',...
    'units', 'normalized',...
    'Position', [.1 .4 .8 .05],...
    'String', 'Start analysis',...
    'CallBack','ica4spmmain');

h_main.save.analysis = uicontrol('style', 'pushbutton',...
    'units', 'normalized',...
    'Position', [.1 .3 .8 .05],...
    'String', 'Save analysis',...
    'CallBack','uisave(''ica'',''ica.mat'')');

h_main.save.designmatrix = uicontrol('style', 'pushbutton',...
    'units', 'normalized',...
    'Position', [.1 .2 .8 .05],...
    'String', 'Specify and save design matrix',...
    'CallBack',['designcomp = listdlg(''PromptString'','...
    '''Specify IC components to save in design matrix:'','...
    '''ListString'',num2str((1:ica.ICs)''));'...
    'X = ica.S(designcomp,:)'';'...
    'uisave(''X'',''X.mat'')']);

h_main.settings.status.panel = uipanel('units', 'normalized',...
    'Position', [.05 .11 .9 .07],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Status');

h_main.settings.status.text = uicontrol('style', 'text',...
    'units', 'normalized',...
    'position', [0.1 .13 .54 .02],...
    'string', 'Ready...',...
    'HorizontalAlignment','left',...    
    'BackgroundColor',[.8 .8 .8],...
    'tooltipstring','Status bar');


h_main.quit = uicontrol('style', 'pushbutton',...
    'units', 'normalized',...
    'Position', [.1 .02 .3 .05],...
    'String', 'Quit',...
    'CallBack','close');

h_main.restart = uicontrol('style', 'pushbutton',...
    'units', 'normalized',...
    'Position', [.6 .02 .3 .05],...
    'String', 'Restart',...
    'CallBack','close all, clear all, ica4spm');
