% =========================================================================
% ICA4SPM toolbox
%
% - IMM, Technical University of Denmark
%
% - version 1.0 
% T. Bjerre, J. Henriksen†, C.H. Nielsen, P.M. Rasmussen
%                    L.K. Hansen, K.H. Madsen
%
% 
%
% Bibtex reference:
%
%  @inproceedings{biosignals2009,
%    author = {Bjerre, T. and Henriksen, J. and Rasmussen, P.M. and Nielsen, C.H. and Hansen, L.K. and Madsen, K.H },
%    title = {Unified ICA-SPM analysis of f{MRI} experiments - Implementation of an {ICA} graphical user interface for the {SPM} pipeline},
%    booktitle = {Proceedings of BIOSTEC - BIOSIGNALS 2009 Conference},
%    year = {2009},}
%
%
% Change log:
% 11 apr 2010 - PMR: User must now supply a mask specifying which voxels to include in the analysis. Minor changes.
% 4 may 2011 - PMR: Minor revisions.
% 
% =========================================================================
function runPCA

global ica h_main st

status = 'Running PCA...';
set(h_main.settings.status.text, 'String', status), drawnow


%% Mask data:

%PMR
% Setup images in columns and make mask:
%
% Find temporal variance components larger than mean  temporal variance:
maske = get(h_main.settings.pca.mask.method,'String');
maske = strtrim(maske(get(h_main.settings.pca.mask.method,'Value'),:));
switch maske
    case 'Variance'
        if size(ica.FileNames,1) > 20   % Limit for variance calculation due to memory reasons
            K = 20;
        else
            K = size(ica.FileNames,1);
        end
        varX = var(Xcolumns(:,1:K),0,2);

        ica.mask = find( varX > 2*mean(varX) );

    case 'Mean'
        ica.mask = find(mean(Xcolumns,2) > 3*mean(Xcolumns(:)));
        
    case 'User specified'
        ica.mask = spm_select(1,'image','Choose file containing (binary) mask');
        dum=spm_read_vols(spm_vol(ica.mask(1:end-2)));
        dum=dum(:);
        ica.mask=find(dum>0.5);   
    case 'None'
        ica.mask = 1:size(Xcolumns,1);
end

 % Get header information etc for images:
ica.Header = spm_vol(ica.FileNames);
ica.Xmasked=zeros(length(ica.mask),length(ica.Header));
% Read in entire image volumes:
for n=1:length(ica.Header)
    dum=spm_read_vols(ica.Header(n));
    dum=dum(:);
    ica.Xmasked(:,n)=dum(ica.mask);
    fprintf('file %d of %d read\n',n,length(ica.Header)),
end
% Finding dimensions of image (4-D):
ica.dimX =[ica.Header(1).dim n];

%% Highpass filter data
K.RT = str2double(ica.TR{1});               % Repetition time
K.row = 1:size(ica.Xmasked,2);  % Time points
K.HParam = 60;                  % Cut-off period [s]
K = spm_filter(K);              % Filter setup
ica.Xmasked= ica.Xmasked';
ica.Xmasked= spm_filter(K,ica.Xmasked ); % Filtering
ica.Xmasked = ica.Xmasked';

%% Run PCA
% Subtract mean:
ica.Xmasked = ica.Xmasked - repmat(mean(ica.Xmasked,2), 1, ica.dimX(4));
% Perform SVD:
[ica.pca.U, ica.pca.S, ica.pca.V]= svd(ica.Xmasked, 0);

status = 'Done running PCA.';
set(h_main.settings.status.text, 'String', status)
